﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Common;
using HeuristicLab.Core;
using System.Drawing;

namespace HeuristicLab.Hive.JobBase {
  [StorableClass]
  public abstract class AbstractJob : DeepCloneable, IJob {

    [Storable]
    protected bool computeInParallel;
    public bool ComputeInParallel {
      get { return computeInParallel; }
      set {
        if (computeInParallel != value) {
          computeInParallel = value;
          OnComputeInParallelChanged();
        }
      }
    }

    [Storable]
    protected bool collectChildJobs;
    public bool CollectChildJobs {
      get { return collectChildJobs; }
      set { collectChildJobs = value; }
    }

    #region constructors and cloning
    public AbstractJob() {

    }
    [StorableConstructor]
    protected AbstractJob(bool deserializing) { }
    protected AbstractJob(AbstractJob original, Cloner cloner)
      : base(original, cloner) {
    }
    #endregion

    #region Event Handlers
    public event EventHandler JobStopped;
    protected virtual void OnJobStopped() {
      EventHandler handler = JobStopped;
      if (handler != null) handler(this, EventArgs.Empty);
    }

    public event EventHandler JobFailed;
    protected virtual void OnJobFailed(EventArgs<Exception> e) {
      EventHandler handler = JobFailed;
      if (handler != null) handler(this, e);
    }

    public event EventHandler<EventArgs<IJob>> NewChildJob;
    protected virtual void OnNewChildJob(IJob job) {
      EventHandler<EventArgs<IJob>> handler = NewChildJob;
      if (handler != null) handler(this, new EventArgs<IJob>(job));
    }

    public event EventHandler WaitForChildJobs;
    protected virtual void OnWaitForChildJobs() {
      EventHandler handler = WaitForChildJobs;
      if (handler != null) handler(this, EventArgs.Empty);
    }

    public event EventHandler DeleteChildJobs;
    protected virtual void OnDeleteChildJobs() {
      EventHandler handler = DeleteChildJobs;
      if (handler != null) handler(this, EventArgs.Empty);
    }

    public event EventHandler ComputeInParallelChanged;
    protected virtual void OnComputeInParallelChanged() {
      EventHandler handler = ComputeInParallelChanged;
      if (handler != null) handler(this, EventArgs.Empty);
    }
    #endregion


    public abstract TimeSpan ExecutionTime { get; }

    public abstract ExecutionState ExecutionState { get; }

    public abstract bool IsParallelizable { get; }

    public abstract void Run();

    public abstract void Prepare();

    public abstract void Start();

    public abstract void Stop();

    public abstract void Resume(IEnumerable<IJob> childJobs);

    public abstract bool CanChangeDescription { get; }

    public abstract bool CanChangeName { get; }

    public abstract string Description { get; set; }

    public abstract string Name { get; set; }

    public abstract event EventHandler NameChanged;
    public abstract event EventHandler<CancelEventArgs<string>> NameChanging;
    public abstract event EventHandler ToStringChanged;
    public abstract event EventHandler DescriptionChanged;

    public abstract string ItemDescription { get; }

    public abstract Image ItemImage { get; }

    public abstract event EventHandler ItemImageChanged;

    public abstract string ItemName { get; }

    public abstract Version ItemVersion { get; }

  }
}
